using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;

namespace TestApp.PropertyGrid_all
{
	#region DynamicPropertyFilterAttribute

   /// <summary>
   /// dlja dinamicheski pokazyvaemogo svojstva
   /// </summary>
   [AttributeUsage(AttributeTargets.Property, Inherited = true)]
   class DynamicPropertyFilterAttribute : Attribute
   {
      string _propertyName;

      /// <summary>
      /// imja svojstva, ot kotorogo zavisit vidimost'  
      /// </summary>
      public string PropertyName
      {
         get { return _propertyName; }
      }

      string _showOn;

      /// <summary>
      /// znachenie svojstva, ot kotorogo zavisit vidimost'  
      /// </summary>
      public string ShowOn
      {
         get { return _showOn; }
      }

      /// <summary>
      /// CTOR  
      /// </summary>
      /// <param name="propName">imja svojstva, ot kotorogo zavisit vidimost'  </param>
      /// <param name="value">znachenie svojstva, ot kotorogo zavisit vidimost'</param>
      public DynamicPropertyFilterAttribute(string propName, string value)
      {
         _propertyName = propName;
         _showOn = value;
      }
   }

	#endregion


   #region FilterablePropertyBase

   /// <summary>
   /// bazovyj klass 
   /// </summary>
   public class FilterablePropertyBase : ICustomTypeDescriptor
   {

      protected PropertyDescriptorCollection GetFilteredProperties(Attribute[] attributes)
      {
         PropertyDescriptorCollection pdc
					= TypeDescriptor.GetProperties(this, attributes, true);
         PropertyDescriptorCollection finalProps = 
					new PropertyDescriptorCollection(new PropertyDescriptor[0]);

         foreach (PropertyDescriptor pd in pdc)
         {
            bool include = false;
            bool Dynamic = false;
            foreach (Attribute a in pd.Attributes)
            {
               if (a is DynamicPropertyFilterAttribute)
               {
                  Dynamic = true;

                  DynamicPropertyFilterAttribute
							dpf = (DynamicPropertyFilterAttribute)a;
                  PropertyDescriptor temp = pdc[dpf.PropertyName];

                  if (dpf.ShowOn.IndexOf(temp.GetValue(this).ToString()) > -1)
                  {
                     include = true;
                  }
               }
            }

            if (!Dynamic || include)
               finalProps.Add(pd);
         }

         return finalProps;
      }

	 #endregion

      #region ICustomTypeDescriptor Members

      public TypeConverter GetConverter()
      {
         return TypeDescriptor.GetConverter(this, true);
      }

      public EventDescriptorCollection GetEvents(Attribute[] attributes)
      {
         return TypeDescriptor.GetEvents(this, attributes, true);
      }

      EventDescriptorCollection ICustomTypeDescriptor.GetEvents()
      {
         return TypeDescriptor.GetEvents(this, true);
      }

      public string GetComponentName()
      {
         return TypeDescriptor.GetComponentName(this, true);
      }

      public object GetPropertyOwner(PropertyDescriptor pd)
      {
         return this;
      }

      public AttributeCollection GetAttributes()
      {
         return TypeDescriptor.GetAttributes(this, true);
      }

      public PropertyDescriptorCollection GetProperties(
         Attribute[] attributes)
      {
         return GetFilteredProperties(attributes);
      }

      PropertyDescriptorCollection ICustomTypeDescriptor.GetProperties()
      {
         return GetFilteredProperties(new Attribute[0]);
      }

      public object GetEditor(Type editorBaseType)
      {
         return TypeDescriptor.GetEditor(this, editorBaseType, true);
      }

      public PropertyDescriptor GetDefaultProperty()
      {
         return TypeDescriptor.GetDefaultProperty(this, true);
      }

      public EventDescriptor GetDefaultEvent()
      {
         return TypeDescriptor.GetDefaultEvent(this, true);
      }

      public string GetClassName()
      {
         return TypeDescriptor.GetClassName(this, true);
      }

      #endregion

   }

}
