#region Module Header

/// /////////////////////////////////////////////////////
/// 1) *** PropertyDescriptor *** - BasisKlasse
///		   ==================
/////////////////////////////////////////////////////////
///		   
/// PropertyGrid arbeitet mit Objekten vom Typ *** PropertyDescriptor *** => wenn man will
/// etwas speziefisches zu machen, muss man eine eigene Klasse fr solche Eigenschaften erzeugen
/// und ableiten die von der Klasse PropertyDescriptor:
/// 
///		public class GlobalizedPropertyDescriptor : PropertyDescriptor 
///	
///	1a) *** PropertyDescriptor *** - Variable
///	Die Klasse muss eine Variable vom Typ *** PropertyDescriptor *** haben.
///	
///	1b) *** ResourceManager *** - Variable
///		    ===============
///		    
///	Wenn man mit Strings aus den Resourcen arbeiten will, muss man eine Variable vom Typ 
///	*** ResourceManager *** haben
///	
////////////////////////////////////////////////////////////////////////////////
///	2) *** ICustomTypeDescriptor ***  / *** PropertyDescriptorCollection ***
///	       =====================			============================
////////////////////////////////////////////////////////////////////////////////
///
///	Diese Klasse enthlt ganze Infos ber alle Eigenschaften vom Typ *** PropertyDescriptor ***.
///	Um einen Zugriff auf die zu haben, muss man eine Variable vom Typ 
///	*** PropertyDescriptorCollection *** deklarieren.
///
/// Hier werden alle Property-Objekte vom Typ GlobalizedPropertyDescriptor erzeugt.
///	Diese Klasse gilt als Container.
///	
////////////////////////////////////////////////////////////////////////////////
///	3) 
////////////////////////////////////////////////////////////////////////////////
///
/// Die Klasse, die alle Eigenschaften fr den PropertyGrid enthllt, muss dann von der 
/// Klasse aus 2) abgeleitet werden:
/// 
///		public class AppSettings  : GlobalizedObject

#endregion

#region using statements

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Reflection;
using System.Globalization;
using System.Resources;

#endregion

namespace TestApp.PropertyGrid_all
{
    #region GlobalizedObject
    /// <summary>
    /// Basisklasse zur Lokalisierung von Controls, z.B. PropertyGrid
    /// </summary>
    public class GlobalizedObject : ICustomTypeDescriptor
    {
        #region Fields

		/// <summary>
		/// PropertyDescriptorCollection
		/// </summary>
        private PropertyDescriptorCollection m_Properties = null;

        #endregion

        #region ICustomTypeDescriptor Implementation

        /// <summary>
        /// bergibt alle die Property Descriptor 
        /// </summary>
        /// <returns>alle Property Descriptoren</returns>
        public PropertyDescriptorCollection GetProperties()
        {
            if(m_Properties == null)
            {
                PropertyDescriptorCollection baseproperties = TypeDescriptor.GetProperties(this, true);
                m_Properties = new PropertyDescriptorCollection(null);

                foreach(PropertyDescriptor pd in baseproperties)
                { m_Properties.Add(new GlobalizedPropertyDescriptor(pd)); }

                m_Properties = m_Properties.Sort(new SortDisplayNames());
            }
            return m_Properties;
        }

        /// <summary>
        /// bergibt die Property Descriptoren eines bestimmten Attributes 
        /// </summary>
        /// <param name="attributes">Auswahl</param>
        /// <returns>bestimmte Property Descriptoren</returns>
        public PropertyDescriptorCollection GetProperties(Attribute[] attributes)
        {
            if (m_Properties == null) 
            {
                PropertyDescriptorCollection baseproperties = TypeDescriptor.GetProperties(this, attributes, true);
                m_Properties = new PropertyDescriptorCollection(null);

                foreach(PropertyDescriptor pd in baseproperties)
                { m_Properties.Add(new GlobalizedPropertyDescriptor(pd)); }

                m_Properties = m_Properties.Sort(new SortDisplayNames());
            }
            return m_Properties;
        }

        #region Aufrufe von statischen Methoden der Default-Type-Infos

        /// <summary>
        /// GetPropertyOwner
        /// </summary>
        /// <param name="pd">PropertyDescriptor</param>
        /// <returns>diese Klasse</returns>
        public object GetPropertyOwner(PropertyDescriptor pd) 
        { return this; }

        /// <summary>
        /// GetClassName
        /// </summary>
        /// <returns>TypeDescriptor.GetClassName</returns>
        public String GetClassName()
        { return TypeDescriptor.GetClassName(this,true); }

        /// <summary>
        /// GetAttributes
        /// </summary>
        /// <returns>TypeDescriptor.GetAttributes</returns>
        public AttributeCollection GetAttributes()
        { return TypeDescriptor.GetAttributes(this,true); }

        /// <summary>
        /// GetComponentName
        /// </summary>
        /// <returns>TypeDescriptor.GetComponentName</returns>
        public String GetComponentName()
        { return TypeDescriptor.GetComponentName(this, true); }

        /// <summary>
        /// GetConverter
        /// </summary>
        /// <returns>TypeDescriptor.GetConverter</returns>
        public TypeConverter GetConverter()
        { return TypeDescriptor.GetConverter(this, true); }

        /// <summary>
        /// GetDefaultEvent
        /// </summary>
        /// <returns>TypeDescriptor.GetDefaultEvent</returns>
        public EventDescriptor GetDefaultEvent() 
        { return TypeDescriptor.GetDefaultEvent(this, true); }

        /// <summary>
        /// GetDefaultProperty
        /// </summary>
        /// <returns>TypeDescriptor.GetDefaultProperty</returns>
        public PropertyDescriptor GetDefaultProperty() 
        { return TypeDescriptor.GetDefaultProperty(this, true); }

        /// <summary>
        /// GetEditor
        /// </summary>
        /// <param name="editorBaseType">Type</param>
        /// <returns>TypeDescriptor.GetEditor</returns>
        public object GetEditor(Type editorBaseType) 
        { return TypeDescriptor.GetEditor(this, editorBaseType, true); }

        /// <summary>
        /// GetEvents Auswahl durch Attribut
        /// </summary>
        /// <param name="attributes">Attribute</param>
        /// <returns>TypeDescriptor.GetEvents</returns>
        public EventDescriptorCollection GetEvents(Attribute[] attributes) 
        { return TypeDescriptor.GetEvents(this, attributes, true); }

        /// <summary>
        /// GetEvents alle
        /// </summary>
        /// <returns>TypeDescriptor.GetEvents</returns>
        public EventDescriptorCollection GetEvents()
        { return TypeDescriptor.GetEvents(this, true); }

        #endregion

        #endregion

        #region Additional Customizations

        /// <summary>
        /// Sets the IsReadOnly Attribute of the Propery 
        /// </summary>
        /// <param name="isReadOnly">The nominal value for the IsReadOnly Attribute of the Property</param>
        /// <param name="propertyName">The name of the Property for which the IsReadOnlyAttribute shall be set.</param>
        public void SetIsReadOnly(bool isReadOnly, string propertyName)
        {
            foreach( GlobalizedPropertyDescriptor gpd in m_Properties )
            {
                if( gpd.Name == propertyName )
                {
                    gpd.SetIsReadOnly(isReadOnly);
                }
            }   
        }

        #endregion

        #region Nested Classes - class SortDisplayNames

        /// <summary>
        /// Hilfsklasse zum Sortieren von GlobalizedPropertyDescriptor Objekten nach OrgDisplayName.
        /// </summary>
        class SortDisplayNames : IComparer
        {
            public int Compare(object x, object y)
            {
                GlobalizedPropertyDescriptor gpd1 = x as GlobalizedPropertyDescriptor;
                GlobalizedPropertyDescriptor gpd2 = y as GlobalizedPropertyDescriptor;

                if(gpd1 == null || gpd2 == null) return 0;

                return gpd1.OrgDisplayName.CompareTo(gpd2.OrgDisplayName);
            }
        }

        #endregion
    }
    
    #endregion

    #region GlobalizedPropertyDescriptor
    /// <summary>
    /// Klasse zur Lokalisierung von Eigenschaften
    /// </summary>
    public class GlobalizedPropertyDescriptor : PropertyDescriptor
    {
        #region Fields

        /// <summary>Basis-Klasse</summary>
        private PropertyDescriptor m_BasePropertyDescriptor; 
        /// <summary>Ressourcen-Manager</summary>
        private ResourceManager    m_RM;

        /// <summary>Kategoriename</summary>
        private string m_Category;
        /// <summary>Anzeigename</summary>
        private string m_DisplayName;
        /// <summary>Beschreibung</summary>
        private string m_Description;

        /// <summary> Is read only attribute for the property</summary>
        private bool m_isReadOnly;

        #endregion

        #region construction 

        /// <summary>
        /// Konstruktor, initilisiert eine neue Instanz der Klasse
        /// </summary>
        /// <param name="basePropertyDescriptor">enthlt den Namen der Eigenschaft und seine die Attribute</param>
        public GlobalizedPropertyDescriptor(PropertyDescriptor basePropertyDescriptor) : base(basePropertyDescriptor)
        {
            m_BasePropertyDescriptor = basePropertyDescriptor;

            m_isReadOnly = basePropertyDescriptor.IsReadOnly;

            string tablename = string.Empty;
                
            tablename += m_BasePropertyDescriptor.ComponentType.Namespace;
            tablename += ".";
            tablename += m_BasePropertyDescriptor.ComponentType.Name;

            try
            {
                m_RM = new ResourceManager(tablename,m_BasePropertyDescriptor.ComponentType.Assembly);
            }
            catch(Exception e)
            {
/*
                PTD.Trace.WriteLine(e.Source, "Localized Property Grid, Load Constructor");
                PTD.Trace.WriteLine(e.Message, "Localized Property Grid, Load Constructor");
                PTD.Trace.WriteLine(e.StackTrace, "Localized Property Grid, Load Constructor");
*/				
            }
        }

        #endregion

        #region overrides and overridables

        /// <summary>
        /// Gibt zurck, ob sich der Wert eines Objekts ndert, wenn das Objekt resetet wird
        /// </summary>
        /// <param name="component">Komponente, die getestet werden soll</param>
        /// <returns>true, wenn sich der Wert beim Reseten ndert</returns>
        public override bool CanResetValue(object component)
        {
            return m_BasePropertyDescriptor.CanResetValue(component);
        }

        /// <summary>
        /// Gibt den Typ zurck, an den das Property gebunden ist
        /// </summary>
        public override Type ComponentType
        {
            get { return m_BasePropertyDescriptor.ComponentType; }
        }

        /// <summary>
        /// Holt den Sprachspezifischen Displayname aus der Ressource
        /// </summary>
        public override string DisplayName
        {
            get 
            {
                if(m_DisplayName == null)
                {
                    string displayname = string.Empty;
                
                    displayname += m_BasePropertyDescriptor.DisplayName;

                    string s = string.Empty;
                    
                    try
                    {
                        s = m_RM.GetString(displayname);
                    }
                    catch(Exception e)
                    {
/*
                        PTD.Trace.WriteLine(e.Source, "Localized Property Grid, Load DisplayName");
                        PTD.Trace.WriteLine(e.Message, "Localized Property Grid, Load DisplayName");
                        PTD.Trace.WriteLine(e.StackTrace, "Localized Property Grid, Load DisplayName");
*/						
                    }
                    m_DisplayName = (s != null && s != string.Empty) ? s : displayname;
                }
                return m_DisplayName;
            }
        }
        /// <summary>
        /// Holt die Sprachspezifische Beschreibung aus der Ressource
        /// </summary>
        public override string Description
        {
            get
            {
                if(m_Description == null)
                {
                    string description = string.Empty;
                
                    description += m_BasePropertyDescriptor.DisplayName;
                    description += ".Description";

                    string s = string.Empty;
                    
                    try
                    {
                        s = m_RM.GetString(description);
                    }
                    catch(Exception e)
                    {
/*
                        PTD.Trace.WriteLine(e.Source, "Localized Property Grid, Load Description");
                        PTD.Trace.WriteLine(e.Message, "Localized Property Grid, Load Description");
                        PTD.Trace.WriteLine(e.StackTrace, "Localized Property Grid, Load Description");
*/						
                    }
                    m_Description = (s != null && s != string.Empty) ? s.Replace(@"\n", "\r\n") : description;
                }
                return m_Description;
            }
        }

        /// <summary>
        /// Holt den Sprachspezifischen Kategorienamen aus der Ressource
        /// </summary>
        public override string Category
        {
            get
            {
                if(m_Category == null)
                {
                    string categoryname = string.Empty;
                    categoryname += m_BasePropertyDescriptor.Category;

                    string s = string.Empty;
                    try
                    {
                        s = m_RM.GetString(categoryname);
                    }
                    catch(Exception e)
                    {
/*
                        PTD.Trace.WriteLine(e.Source, "Localized Property Grid, Load Category");
                        PTD.Trace.WriteLine(e.Message, "Localized Property Grid, Load Category");
                        PTD.Trace.WriteLine(e.StackTrace, "Localized Property Grid, Load Category");
*/						
                    }
                    m_Category = (s != null && s != string.Empty) ? s : categoryname;
                }
                return m_Category;
            }
        }
        
        /// <summary>
        /// gibt den momentanen Wert der Property einer Komponente zurck
        /// </summary>
        /// <param name="component">Komponente, zu welcher der Wert ausgelesen werden soll</param>
        /// <returns>Wert der Property</returns>
        public override object GetValue(object component)
        { return m_BasePropertyDescriptor.GetValue(component); }

       
        /// <summary>
        /// Resetet den Wert des Properties auf "Default"-Wert
        /// </summary>
        /// <param name="component">zu behandelnde Komponente</param>
        public override void ResetValue(object component)
        { m_BasePropertyDescriptor.ResetValue(component); }
    
        /// <summary>
        /// zeigt an, ob ein Property persistent ist
        /// </summary>
        /// <param name="component">zu behandelnde Komponente</param>
        /// <returns>true, wenn Property persistent ist</returns>
        public override bool ShouldSerializeValue(object component)
        { return m_BasePropertyDescriptor.ShouldSerializeValue(component); }

        /// <summary>
        /// setzt den Wert eines Properties
        /// </summary>
        /// <param name="component">zu behandelnde Komponente</param>
        /// <param name="value">neuer Wert</param>
        public override void SetValue(object component, object value)
        { m_BasePropertyDescriptor.SetValue(component, value); }

        #endregion

        #region overriden properties

        /// <summary>
        /// gibt zurck, ob das Property schreibbar oder readonly ist
        /// </summary>
        public override bool IsReadOnly
        {
            get { return m_isReadOnly; /*m_BasePropertyDescriptor.IsReadOnly;*/ } 
        }

        /// <summary>
        /// Name der Property
        /// </summary>
        public override string Name
        { get { return m_BasePropertyDescriptor.Name; } }

        /// <summary>
        /// Typ des Property
        /// </summary>
        public override Type PropertyType
        { get { return m_BasePropertyDescriptor.PropertyType; } }

        #endregion

        #region Additional Customization

        /// <summary>
        /// gibt den Original-Namen fr die Sortierung zurck
        /// </summary>
        public string OrgDisplayName
        {
            get
            {
                return m_BasePropertyDescriptor.DisplayName;
            }
        }

        /// <summary>
        /// Sets the IsReadOnly Flag for this Property
        /// </summary>
        /// <param name="isReadOnly"></param>
        public void SetIsReadOnly(bool isReadOnly)
        {
            m_isReadOnly = isReadOnly;
        }

        #endregion
    }

    #endregion
}
