using System;

using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using System.Data;
using System.Globalization; 

namespace TestApp.PropertyGrid_all
{

	#region	SpellingOptionsConverter

	#region	Modulkopf: SpellingOptionsConverter
	/// <summary>
	/// Class: SpellingOptionsConverter
	/// 
	/// TypeConverter 
	/// 
	/// </summary>
	#endregion

	public class SpellingOptionsConverter : ExpandableObjectConverter 
	{  

		/// <summary>
		/// prfen, ob es bei der Konvertierung um unsere Klasse sich handelt
		/// </summary>
		/// <param name="context"></param>
		/// <param name="destinationType"></param>
		/// <returns></returns>
		public override bool CanConvertTo(	ITypeDescriptorContext context,
			System.Type destinationType) 
		{
			if (destinationType == typeof(SpellingOptions))
			{
				return true;
			}

			return base.CanConvertTo(context, destinationType);
		}

		/// <summary>
		/// Konvertieren das SpellingOptions-Objekt in ein String-Objekt fr unseren Grid (fr die Anzeige) 
		/// wenn Im Grig ein Objekt vom Typ String erwartet ist
		/// </summary>
		/// <param name="context"></param>
		/// <param name="culture"></param>
		/// <param name="value"></param>
		/// <param name="destinationType"></param>
		/// <returns></returns>
		public override object ConvertTo(	ITypeDescriptorContext context,
			CultureInfo culture, 
			object value, 
			System.Type destinationType) 
		{
			if (destinationType == typeof(System.String) && value is SpellingOptions)
			{
				SpellingOptions so = (SpellingOptions)value;

				return "berprfung whrend der Eingabe:" + String.Format("{0}", so.SpellCheckWhileTyping) + 
					", GROSSBUCHSTABEN berprfen: " + String.Format("{0}", so.SpellCheckCAPS) +
					", Korrekturen vorschlagen: " + String.Format("{0}", so.SuggestCorrections);
			}
			return base.ConvertTo(context, culture, value, destinationType);
		}

		/// <summary>
		/// Konvertieren das String-Objekt aus dem Grid in ein SpellingOptions-Objekt
		/// </summary>
		/// <param name="context"></param>
		/// <param name="culture"></param>
		/// <param name="value"></param>
		/// <returns></returns>
		public override object ConvertFrom(	ITypeDescriptorContext context,
											CultureInfo culture, 
											object value) 
		{
			if (value is string) 
			{
				try 
				{
					string s = (string) value;
					int colon = s.IndexOf(':');
					int comma = s.IndexOf(',');
					int nDiff = 0;
					if (colon == -1 || comma == -1) 
					{
						nDiff = 0;
					}
					else
					{
						nDiff = comma - colon - 1;
					}

					{
						string checkWhileTyping = s.Substring(colon + 1 ,
							(nDiff));

						colon = s.IndexOf(':', comma + 1);
						comma = s.IndexOf(',', comma + 1);
						if (colon == -1 || comma == -1) 
						{
							nDiff = 0;
						}
						else
						{
							nDiff = comma - colon - 1;
						}
						string checkCaps = s.Substring(colon + 1 , 
							(nDiff));
						colon = s.IndexOf(':', comma + 1);
						string suggCorr = s.Substring(colon + 1);
						SpellingOptions so = new SpellingOptions();

						if (checkWhileTyping != "")
							so.SpellCheckWhileTyping =true;
						else
							so.SpellCheckWhileTyping = false;

						if (checkCaps != "")
							so.SpellCheckCAPS = true;
						else
							so.SpellCheckCAPS = false;

						if (suggCorr != "")
							so.SuggestCorrections = true;
						else
							so.SuggestCorrections = false;

						return so;
					}
				}
				catch 
				{
					throw new ArgumentException(
						" '" + (string)value + "kann nicht " +  
						"' in Typ SpellingOptions konvertiert werden");
				}
			}  
			return base.ConvertFrom(context, culture, value);
		}



		/// <summary>
		/// prfen, ob wir berhaupt konvertieren drfen - das gilt fr die 
		/// Eigenschaft-Bearbeitung im PropertyGrid (TypeConverterAttribute oder BrowsableAttribute oder ReadOnlyAttribute ???)
		/// </summary>
		/// <param name="context"></param>
		/// <param name="sourceType"></param>
		/// <returns></returns>
		public override bool CanConvertFrom(ITypeDescriptorContext context,
			System.Type sourceType) 
		{
			if (sourceType == typeof(string))
			{
				return true;
			}

			return base.CanConvertFrom(context, sourceType);
		}

	}

	#endregion


	#region SpellingOptions

	/// <summary>
	/// Summary description for SpellingOptions.
	/// </summary>
	/// 

	[TypeConverterAttribute(typeof(SpellingOptionsConverter)),
	 DescriptionAttribute("Erweitern, um die Rechtschreiboptionen fr die Anwendung anzuzeigen.")]

	public class SpellingOptions
	{
		public SpellingOptions()
		{
			//
			// TODO: Add constructor logic here
			//
		}

		private bool spellCheckWhileTyping = true;
		private bool spellCheckCAPS = false;
		private bool suggestCorrections = true;

		[DefaultValueAttribute(true)]
		public bool SpellCheckWhileTyping 
		{
			get { return spellCheckWhileTyping; }
			set { spellCheckWhileTyping = value; }
		}

		[DefaultValueAttribute(false)]
		public bool SpellCheckCAPS 
		{
			get { return spellCheckCAPS; }
			set { spellCheckCAPS = value; }
		}

		[DefaultValueAttribute(true)]
		public bool SuggestCorrections 
		{
			get { return suggestCorrections; }
			set { suggestCorrections = value; }
		}

	}

	#endregion
}


